@@ -2,30 +2,7 @@
 Atomica root module
 
 The Atomica module is the entry point for running simulations and performing analysis
-using Atomica. It consists of the following submodules:
-
-    .. autosummary::
-        :toctree: _autosummary
-
-        calibration
-        cascade
-        data
-        defaults
-        excel
-        framework
-        migration
-        model
-        optimization
-        parameters
-        function_parser
-        plotting
-        programs
-        project
-        reconciliation
-        results
-        scenarios
-        system
-        utils
+using Atomica.
 
 """
 
@@ -48,7 +25,8 @@ using Atomica. It consists of the following submodules:
 
 import sys
 import logging
-logger = logging.getLogger('atomica')
+
+logger = logging.getLogger("atomica")
 
 if not logger.hasHandlers():
     # Only add handlers if they don't already exist in the module-level logger
@@ -67,38 +45,44 @@ if not logger.hasHandlers():
     debug_handler.addFilter(type("ThresholdFilter", (object,), {"filter": lambda x, logRecord: logRecord.levelno < logging.INFO})())  # Display anything INFO or higher
     info_handler.addFilter(type("ThresholdFilter", (object,), {"filter": lambda x, logRecord: logRecord.levelno < logging.WARNING})())  # Don't display WARNING or higher
 
-    debug_formatter = logging.Formatter('%(levelname)s {%(filename)s:%(lineno)d} - %(message)s')
+    debug_formatter = logging.Formatter("%(levelname)s {%(filename)s:%(lineno)d} - %(message)s")
     debug_handler.setFormatter(debug_formatter)
 
     logger.addHandler(debug_handler)
     logger.addHandler(info_handler)
     logger.addHandler(warning_handler)
-    logger.setLevel('INFO')  # Set the overall log level
+    logger.setLevel("INFO")  # Set the overall log level
 
 from .version import version as __version__, versiondate as __versiondate__, gitinfo as __gitinfo__
 
 # Check scipy version
 import scipy
 import sciris as sc
-if sc.compareversions(scipy.__version__, '1.2.1') < 0:
-    raise Exception(f'Atomica requires Scipy 1.2.1 or later - installed version is {scipy.__version__}')
+
+if sc.compareversions(scipy.__version__, "1.2.1") < 0:
+    raise Exception(f"Atomica requires Scipy 1.2.1 or later - installed version is {scipy.__version__}")
 
 # Increase Framework performance by not calling garbage collection all the time
 import pandas as pd
-pd.set_option('mode.chained_assignment', None)
+
+pd.set_option("mode.chained_assignment", None)
 
 # The Atomica user interface -- import from submodules
-from .framework import *
-from .project import *
 from .calibration import *
-from .scenarios import *
-from .defaults import *
+from .cascade import *
+from .data import *
+from .demos import *
+from .framework import *
+from .function_parser import *
+from .migration import migrations, register_migration
+from .model import *
+from .optimization import *
+from .parameters import *
 from .plotting import *
 from .programs import *
+from .project import *
 from .reconciliation import *
-from .optimization import *
-from .cascade import *
 from .results import *
-from .migration import migrations, register_migration
-from .utils import *
+from .scenarios import *
 from .system import *
+from .utils import *
