@@ -20,7 +20,9 @@ from . import logger
 from .system import FrameworkSettings as FS
 from collections import defaultdict
 
-_DEFAULT_PROVENANCE = 'Framework-supplied default'
+_DEFAULT_PROVENANCE = "Framework-supplied default"
+
+__all__ = ["ProjectData"]
 
 
 class ProjectData(sc.prettyobj):
@@ -156,8 +158,8 @@ class ProjectData(sc.prettyobj):
 
         # Then, if the key is None, we are working on a transfer parameter. So reconstruct the key
         if key is None:
-            x = name.split('_to_')
-            code_name, from_pop = x[0].split('_', 1)
+            x = name.split("_to_")
+            code_name, from_pop = x[0].split("_", 1)
             to_pop = x[1]
             name = code_name
             key = (from_pop, to_pop)
@@ -207,25 +209,25 @@ class ProjectData(sc.prettyobj):
 
         if sc.isnumber(pops):
             for i in range(0, pops):
-                new_pops['pop_%d' % (i)] = {'label': 'Population %d' % (i), 'type': default_pop_type}
+                new_pops["pop_%d" % (i)] = {"label": "Population %d" % (i), "type": default_pop_type}
         else:
             for code_name, spec in pops.items():
                 if sc.isstring(spec):
-                    new_pops[code_name] = {'label': spec, 'type': default_pop_type}
+                    new_pops[code_name] = {"label": spec, "type": default_pop_type}
                 else:
                     new_pops[code_name] = spec
 
         if not new_pops:
-            raise Exception('A new databook must have at least 1 population')
+            raise Exception("A new databook must have at least 1 population")
 
         new_transfers = sc.odict()
         if sc.isnumber(transfers):
             for i in range(0, transfers):
-                new_transfers['transfer_%d' % (i)] = {'label': 'Transfer %d' % (i), 'type': default_pop_type}
+                new_transfers["transfer_%d" % (i)] = {"label": "Transfer %d" % (i), "type": default_pop_type}
         else:
             for code_name, spec in transfers.items():
                 if sc.isstring(spec):
-                    new_transfers[code_name] = {'label': spec, 'type': default_pop_type}
+                    new_transfers[code_name] = {"label": spec, "type": default_pop_type}
                 else:
                     new_transfers[code_name] = spec
 
@@ -234,54 +236,54 @@ class ProjectData(sc.prettyobj):
         data.tvec = sc.promotetoarray(tvec)
         pages = defaultdict(list)  # This will store {sheet_name:(code_name,databook_order)} which will then get sorted further
 
-        for obj_type, df in zip(['comps', 'characs', 'pars'], [framework.comps, framework.characs, framework.pars]):
+        for obj_type, df in zip(["comps", "characs", "pars"], [framework.comps, framework.characs, framework.pars]):
             for _, spec in df.iterrows():
-                databook_page = spec.get('databook page')
+                databook_page = spec.get("databook page")
                 if databook_page is not None:
-                    pop_type = spec.get('population type')
-                    databook_order = spec.get('databook order')
-                    full_name = spec['display name']
+                    pop_type = spec.get("population type")
+                    databook_order = spec.get("databook order")
+                    full_name = spec["display name"]
 
                     if databook_order is None:
                         order = np.inf
                     else:
                         order = databook_order
                     pages[databook_page].append((spec.name, order))
-                    data.tdve[spec.name] = TimeDependentValuesEntry(full_name, data.tvec, allowed_units=[framework.get_databook_units(full_name)], comment=spec['guidance'])
+                    data.tdve[spec.name] = TimeDependentValuesEntry(full_name, data.tvec, allowed_units=[framework.get_databook_units(full_name)], comment=spec["guidance"])
                     data.tdve[spec.name].write_units = True
                     data.tdve[spec.name].write_uncertainty = True
-                    if obj_type == 'pars':
+                    if obj_type == "pars":
                         data.tdve[spec.name].write_assumption = True
-                        if spec['timed'] == 'y':
+                        if spec["timed"] == "y":
                             data.tdve[spec.name].tvec = []  # If parameter is timed, don't show any years
                             data.tdve[spec.name].write_uncertainty = False  # Don't show uncertainty for timed parameters. In theory users could manually add the column and sample over it, but because the duration is rounded to the timestep, it's likely to have confusing stepped effects
                     data.tdve[spec.name].pop_type = pop_type
 
         # Now convert pages to full names and sort them into the correct order
-        for _, spec in framework.sheets['databook pages'][0].iterrows():
+        for _, spec in framework.sheets["databook pages"][0].iterrows():
 
-            if spec['datasheet code name'] in pages:
-                pages[spec['datasheet code name']].sort(key=lambda x: x[1])
-                data.tdve_pages[spec['datasheet title']] = [x[0] for x in pages[spec['datasheet code name']]]
+            if spec["datasheet code name"] in pages:
+                pages[spec["datasheet code name"]].sort(key=lambda x: x[1])
+                data.tdve_pages[spec["datasheet title"]] = [x[0] for x in pages[spec["datasheet code name"]]]
             else:
-                data.tdve_pages[spec['datasheet title']] = list()
+                data.tdve_pages[spec["datasheet title"]] = list()
 
         # Now, proceed to add pops, transfers, and interactions
         for code_name, spec in new_pops.items():
-            data.add_pop(code_name, spec['label'], pop_type=spec['type'])
+            data.add_pop(code_name, spec["label"], pop_type=spec["type"])
 
         for code_name, spec in new_transfers.items():
-            data.add_transfer(code_name, spec['label'], pop_type=spec['type'])
+            data.add_transfer(code_name, spec["label"], pop_type=spec["type"])
 
         for _, spec in framework.interactions.iterrows():
-            interpop = data.add_interaction(spec.name, spec['display name'], from_pop_type=spec['from population type'], to_pop_type=spec['to population type'])
-            if 'default value' in spec and spec['default value']:
+            interpop = data.add_interaction(spec.name, spec["display name"], from_pop_type=spec["from population type"], to_pop_type=spec["to population type"])
+            if "default value" in spec and spec["default value"]:
                 for from_pop in interpop.from_pops:
                     for to_pop in interpop.to_pops:
                         ts = TimeSeries(units=interpop.allowed_units[0])
-                        ts.insert(None, spec['default value'])
+                        ts.insert(None, spec["default value"])
                         interpop.ts[(from_pop, to_pop)] = ts
-                        interpop.ts_attributes['Provenance'][(from_pop, to_pop)] = _DEFAULT_PROVENANCE
+                        interpop.ts_attributes["Provenance"][(from_pop, to_pop)] = _DEFAULT_PROVENANCE
 
         # Finally, insert parameter and characteristic default values
         for df in [framework.comps, framework.characs, framework.pars]:
@@ -289,11 +291,11 @@ class ProjectData(sc.prettyobj):
                 # In order to write a default value
                 # - The default value should be present and not None
                 # - The quantity should appear in the databook
-                if 'default value' in spec and (spec['default value'] is not None) and spec['databook page']:
+                if "default value" in spec and (spec["default value"] is not None) and spec["databook page"]:
                     tdve = data.tdve[spec.name]
                     for key, ts in tdve.ts.items():
-                        ts.insert(None, spec['default value'])
-                        tdve.ts_attributes['Provenance'][key] = _DEFAULT_PROVENANCE
+                        ts.insert(None, spec["default value"])
+                        tdve.ts_attributes["Provenance"][key] = _DEFAULT_PROVENANCE
 
         return data
 
@@ -326,7 +328,7 @@ class ProjectData(sc.prettyobj):
             spreadsheet = sc.Spreadsheet(spreadsheet)
 
         workbook = openpyxl.load_workbook(spreadsheet.tofile(), read_only=True, data_only=True)  # Load in read-only mode for performance, since we don't parse comments etc.
-        validate_category(workbook, 'atomica:databook')
+        validate_category(workbook, "atomica:databook")
 
         # These sheets are optional - if none of these are provided in the databook
         # then they will remain empty
@@ -335,28 +337,28 @@ class ProjectData(sc.prettyobj):
 
         for sheet in workbook.worksheets:
 
-            if sheet.title.startswith('#ignore'):
+            if sheet.title.startswith("#ignore"):
                 continue
 
-            if sheet.title == 'Population Definitions':
+            if sheet.title == "Population Definitions":
                 try:
                     self._read_pops(sheet)
                 except Exception as e:
                     message = 'An error was detected on the "Population Definitions" sheet'
-                    raise Exception('%s -> %s' % (message, e)) from e
-            elif sheet.title == 'Transfers':
+                    raise Exception("%s -> %s" % (message, e)) from e
+            elif sheet.title == "Transfers":
                 try:
                     self._read_transfers(sheet)
                 except Exception as e:
                     message = 'An error was detected on the "Transfers" sheet'
-                    raise Exception('%s -> %s' % (message, e)) from e
-            elif sheet.title == 'Interactions':
+                    raise Exception("%s -> %s" % (message, e)) from e
+            elif sheet.title == "Interactions":
                 try:
                     self._read_interpops(sheet)
                 except Exception as e:
                     message = 'An error was detected on the "Interactions" sheet'
-                    raise Exception('%s -> %s' % (message, e)) from e
-            elif sheet.title == 'Metadata':
+                    raise Exception("%s -> %s" % (message, e)) from e
+            elif sheet.title == "Metadata":
                 continue
             else:
                 self.tdve_pages[sheet.title] = []
@@ -367,7 +369,7 @@ class ProjectData(sc.prettyobj):
                         tdve = TimeDependentValuesEntry.from_rows(table)
                     except Exception as e:
                         message = 'Error on sheet "%s" while trying to read a TDVE table starting on row %d' % (sheet.title, start_row)
-                        raise Exception('%s -> %s' % (message, e)) from e
+                        raise Exception("%s -> %s" % (message, e)) from e
 
                     # If the TDVE is not in the Framework, that's a critical stop error, because the framework needs to at least declare
                     # what kind of variable this is - otherwise, we don't know the allowed units and cannot write the databook back properly
@@ -379,7 +381,7 @@ class ProjectData(sc.prettyobj):
 
                     code_name = spec.name
                     tdve.allowed_units = [framework.get_databook_units(code_name)]
-                    tdve.pop_type = spec['population type']
+                    tdve.pop_type = spec["population type"]
 
                     # Migrate the units (20181114)
                     # All TimeSeries instances in databook TDVE tables should have the same units as the allowed units
@@ -390,9 +392,9 @@ class ProjectData(sc.prettyobj):
                             if not ts.units or ts.units.strip().lower() == tdve.allowed_units[0].strip().split()[0].strip().lower():
                                 ts.units = tdve.allowed_units[0]
 
-                    if not spec['databook page']:
+                    if not spec["databook page"]:
                         logger.warning('A TDVE table for "%s" (%s) was read in and will be used, but the Framework did not mark this quantity as appearing in the databook', tdve.name, code_name)
-                    tdve.comment = spec['guidance']
+                    tdve.comment = spec["guidance"]
 
                     if code_name in self.tdve:
                         raise Exception('A TDVE table for "%s" (%s) appears more than once in the databook. The first table was on sheet "%s" and the first duplicate table is on sheet "%s" starting on row %d' % (tdve.name, code_name, [k for k, v in self.tdve_pages.items() if code_name in v][0], sheet.title, start_row))
@@ -441,26 +443,26 @@ class ProjectData(sc.prettyobj):
         # Make sure that all of the quantities the Framework says we should read in have been read in, and that
         # those quantities all have some data values associated with them
         for pop in self.pops.values():
-            if pop['type'] is None:
-                pop['type'] = self._pop_types[0]
-            assert pop['type'] in self._pop_types, 'Error in population "%s": population type "%s" not found in framework. If the framework defines a non-default population type, then it must be explicitly specified in databooks and program books.' % (pop['label'], pop['type'])
+            if pop["type"] is None:
+                pop["type"] = self._pop_types[0]
+            assert pop["type"] in self._pop_types, 'Error in population "%s": population type "%s" not found in framework. If the framework defines a non-default population type, then it must be explicitly specified in databooks and program books.' % (pop["label"], pop["type"])
 
-        for obj_type, df in zip(['comps', 'characs', 'pars'], [framework.comps, framework.characs, framework.pars]):
-            for spec_name, spec in zip(df.index, df.to_dict(orient='records')):
+        for obj_type, df in zip(["comps", "characs", "pars"], [framework.comps, framework.characs, framework.pars]):
+            for spec_name, spec in zip(df.index, df.to_dict(orient="records")):
                 if spec_name in self.pops:
                     raise Exception('Code name "%s" has been used for both a population and a framework quantity - population names must be unique' % (spec_name))
 
-                if spec['databook page'] is not None:
+                if spec["databook page"] is not None:
                     if spec_name not in self.tdve:
-                        if spec['default value'] is None:
-                            raise Exception('The databook did not contain a required TDVE table named "%s" (code name "%s")' % (spec['display name'], spec_name))
+                        if spec["default value"] is None:
+                            raise Exception('The databook did not contain a required TDVE table named "%s" (code name "%s")' % (spec["display name"], spec_name))
                         else:
-                            logger.warning('TDVE table "%s" (code name "%s") is missing from the databook. Using default values from the framework' % (spec['display name'], spec_name))
+                            logger.warning('TDVE table "%s" (code name "%s") is missing from the databook. Using default values from the framework' % (spec["display name"], spec_name))
                             units = framework.get_databook_units(spec_name)
-                            self.tdve[spec_name] = TimeDependentValuesEntry(spec['display name'], self.tvec.copy(), allowed_units=[units], comment=spec['guidance'])
+                            self.tdve[spec_name] = TimeDependentValuesEntry(spec["display name"], self.tvec.copy(), allowed_units=[units], comment=spec["guidance"])
                             for pop in self.pops.keys():
-                                self.tdve[spec_name].ts[pop] = TimeSeries(assumption=spec['default value'], units=units)
-                            tdve_page = framework.sheets['databook pages'][0][framework.sheets['databook pages'][0]['datasheet code name'] == spec['databook page']]['datasheet title'].values[0]
+                                self.tdve[spec_name].ts[pop] = TimeSeries(assumption=spec["default value"], units=units)
+                            tdve_page = framework.sheets["databook pages"][0][framework.sheets["databook pages"][0]["datasheet code name"] == spec["databook page"]]["datasheet title"].values[0]
                             if tdve_page in self.tdve_pages:
                                 self.tdve_pages[tdve_page].append(spec_name)
                             else:
@@ -472,21 +474,21 @@ class ProjectData(sc.prettyobj):
                         location = 'Error in TDVE table "%s" on sheet "%s"' % (tdve.name, tdve_sheet)
                         assert tdve.pop_type in self._pop_types, '%s. Population type "%s" did not match any in the framework' % (location, tdve.pop_type)
 
-                        required_pops = [x for x, y in self.pops.items() if y['type'] == tdve.pop_type]  # The TDVE should contain values for all populations of that type, otherwise cannot construct the ParameterSet. Check that these populations are all present
+                        required_pops = [x for x, y in self.pops.items() if y["type"] == tdve.pop_type]  # The TDVE should contain values for all populations of that type, otherwise cannot construct the ParameterSet. Check that these populations are all present
                         missing_pops = set(required_pops).difference(tdve.ts.keys())
                         if missing_pops:
                             raise Exception("%s. The following populations were not supplied but are required: %s" % (location, missing_pops))
 
                         for name, ts in self.tdve[spec_name].ts.items():
-                            assert ts.has_data, '%s. Data values missing for %s (%s)' % (location, tdve.name, name)
-                            assert ts.units is not None, '%s. Units missing for %s (%s)' % (location, tdve.name, name)
+                            assert ts.has_data, "%s. Data values missing for %s (%s)" % (location, tdve.name, name)
+                            assert ts.units is not None, "%s. Units missing for %s (%s)" % (location, tdve.name, name)
                             if ts.units.strip().lower() != framework_units.strip().lower():
                                 # If the units don't match the framework's 'databook' units, see if they at least match the standard unit (for legacy databooks)
                                 # For compartments and characteristics, the units must match exactly
-                                if obj_type in ['comps', 'characs'] or ('format' in spec and spec['format'] is not None and ts.units.lower().strip() != spec['format'].lower().strip()):
+                                if obj_type in ["comps", "characs"] or ("format" in spec and spec["format"] is not None and ts.units.lower().strip() != spec["format"].lower().strip()):
                                     assert ts.units == framework_units, '%s. Unit "%s" for %s (%s) does not match the declared units from the Framework (expecting "%s")' % (location, ts.units, tdve.name, name, framework_units)
-                            if obj_type == 'par' and spec['timed'] == 'y':
-                                assert not ts.has_time_data, '%s. Parameter %s (%s) is marked as a timed transition in the Framework, so it must have a constant value (i.e., the databook cannot contain time-dependent values for this parameter)' % (location, tdve.name, name)
+                            if obj_type == "par" and spec["timed"] == "y":
+                                assert not ts.has_time_data, "%s. Parameter %s (%s) is marked as a timed transition in the Framework, so it must have a constant value (i.e., the databook cannot contain time-dependent values for this parameter)" % (location, tdve.name, name)
 
         for tdc in self.interpops + self.transfers:
             if tdc.from_pop_type is None:  # Supply default pop type
@@ -501,10 +503,10 @@ class ProjectData(sc.prettyobj):
                 if tdc.code_name == spec.name:
                     for (from_pop, to_pop), ts in tdc.ts.items():
                         assert to_pop in self.pops, 'Population "%s" in "%s" not recognized. Should be one of: %s' % (to_pop, spec.name, self.pops.keys())
-                        assert self.pops[to_pop]['type'] == tdc.to_pop_type, 'Interaction "%s" has to-population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.to_pop_type, to_pop, self.pops[to_pop]['type'])
+                        assert self.pops[to_pop]["type"] == tdc.to_pop_type, 'Interaction "%s" has to-population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.to_pop_type, to_pop, self.pops[to_pop]["type"])
                         assert from_pop in self.pops, 'Population "%s" in "%s" not recognized. Should be one of: %s' % (from_pop, spec.name, self.pops.keys())
-                        assert self.pops[from_pop]['type'] == tdc.from_pop_type, 'Interaction "%s" has from-population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.from_pop_type, from_pop, self.pops[from_pop]['type'])
-                        assert ts.has_data, 'Data values missing for interaction %s, %s->%s' % (spec.name, to_pop, from_pop)
+                        assert self.pops[from_pop]["type"] == tdc.from_pop_type, 'Interaction "%s" has from-population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.from_pop_type, from_pop, self.pops[from_pop]["type"])
+                        assert ts.has_data, "Data values missing for interaction %s, %s->%s" % (spec.name, to_pop, from_pop)
                         assert ts.units.lower().title() == FS.DEFAULT_SYMBOL_INAPPLICABLE.lower().title(), 'Units error in interaction %s, %s->%s. Interaction units must be "N.A."' % (spec.name, to_pop, from_pop)
                     break
             else:
@@ -513,11 +515,11 @@ class ProjectData(sc.prettyobj):
         for tdc in self.transfers:
             for (from_pop, to_pop), ts in tdc.ts.items():
                 assert to_pop in self.pops, 'Population "%s" in "%s" not recognized. Should be one of: %s' % (to_pop, tdc.full.name, self.pops.keys())
-                assert self.pops[to_pop]['type'] == tdc.to_pop_type, 'Transfer "%s" has population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.to_pop_type, to_pop, self.pops[to_pop]['type'])
+                assert self.pops[to_pop]["type"] == tdc.to_pop_type, 'Transfer "%s" has population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.to_pop_type, to_pop, self.pops[to_pop]["type"])
                 assert from_pop in self.pops, 'Population "%s" in "%s" not recognized. Should be one of: %s' % (from_pop, tdc.full.name, self.pops.keys())
-                assert self.pops[from_pop]['type'] == tdc.from_pop_type, 'Transfer "%s" has population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.from_pop_type, from_pop, self.pops[from_pop]['type'])
-                assert ts.has_data, 'Data values missing for transfer %s, %s->%s' % (tdc.full_name, to_pop, from_pop)
-                assert ts.units is not None, 'Units are missing for transfer %s, %s->%s' % (tdc.full_name, to_pop, from_pop)
+                assert self.pops[from_pop]["type"] == tdc.from_pop_type, 'Transfer "%s" has population type "%s", but contains Population "%s", which is type "%s"' % (tdc.full_name, tdc.from_pop_type, from_pop, self.pops[from_pop]["type"])
+                assert ts.has_data, "Data values missing for transfer %s, %s->%s" % (tdc.full_name, to_pop, from_pop)
+                assert ts.units is not None, "Units are missing for transfer %s, %s->%s" % (tdc.full_name, to_pop, from_pop)
         return True
 
     def to_workbook(self) -> tuple:
@@ -537,11 +539,11 @@ class ProjectData(sc.prettyobj):
 
         # Initialize the bytestream
         f = io.BytesIO()
-        wb = xw.Workbook(f, {'in_memory': True})
+        wb = xw.Workbook(f, {"in_memory": True})
 
         # Open a workbook
         self._book = wb
-        self._book.set_properties({'category': 'atomica:databook'})
+        self._book.set_properties({"category": "atomica:databook"})
         self._formats = standard_formats(self._book)
         self._references = {}  # Reset the references dict
 
@@ -608,7 +610,7 @@ class ProjectData(sc.prettyobj):
         if code_name.lower() in FS.RESERVED_KEYWORDS:
             raise Exception('Population name "%s" is a reserved keyword' % (code_name.lower()))
 
-        self.pops[code_name] = {'label': full_name, 'type': pop_type}
+        self.pops[code_name] = {"label": full_name, "type": pop_type}
 
         for interaction in self.transfers + self.interpops:
             if interaction.from_pop_type == pop_type:
@@ -647,7 +649,7 @@ class ProjectData(sc.prettyobj):
         self.pops.rename(existing_code_name, new_code_name)
 
         # Then change the full name
-        self.pops[new_code_name]['label'] = new_full_name
+        self.pops[new_code_name]["label"] = new_full_name
 
         # Update interactions and transfers - need to change all of the to/from tuples
         for interaction in self.transfers + self.interpops:
@@ -701,15 +703,15 @@ class ProjectData(sc.prettyobj):
         if pop_type is None:
             pop_type = self._pop_types[0]
 
-        assert pop_type in self._pop_types, 'Population type %s not found in framework' % (pop_type)
+        assert pop_type in self._pop_types, "Population type %s not found in framework" % (pop_type)
 
         for transfer in self.transfers:
             assert code_name != transfer.code_name, 'Transfer with name "%s" already exists' % (code_name)
 
-        pop_names = [name for name, pop_spec in self.pops.items() if pop_spec['type'] == pop_type]
+        pop_names = [name for name, pop_spec in self.pops.items() if pop_spec["type"] == pop_type]
 
         # Here, need to list all relevant populations
-        new_transfer = TimeDependentConnections(code_name, full_name, self.tvec, from_pops=pop_names, to_pops=pop_names, interpop_type='transfer', ts=None, from_pop_type=pop_type, to_pop_type=pop_type)
+        new_transfer = TimeDependentConnections(code_name, full_name, self.tvec, from_pops=pop_names, to_pops=pop_names, interpop_type="transfer", ts=None, from_pop_type=pop_type, to_pop_type=pop_type)
         new_transfer.write_units = True
         new_transfer.write_assumption = True
         new_transfer.write_uncertainty = True
@@ -776,15 +778,15 @@ class ProjectData(sc.prettyobj):
         if to_pop_type is None:
             to_pop_type = self._pop_types[0]
 
-        assert from_pop_type in self._pop_types, 'Population type %s not found in framework' % (from_pop_type)
-        assert to_pop_type in self._pop_types, 'Population type %s not found in framework' % (to_pop_type)
+        assert from_pop_type in self._pop_types, "Population type %s not found in framework" % (from_pop_type)
+        assert to_pop_type in self._pop_types, "Population type %s not found in framework" % (to_pop_type)
 
         for interaction in self.interpops:
             assert code_name != interaction.code_name, 'Interaction with name "%s" already exists' % (code_name)
 
-        from_pops = [name for name, pop_spec in self.pops.items() if pop_spec['type'] == from_pop_type]
-        to_pops = [name for name, pop_spec in self.pops.items() if pop_spec['type'] == to_pop_type]
-        interpop = TimeDependentConnections(code_name, full_name, tvec=self.tvec, from_pops=from_pops, to_pops=to_pops, interpop_type='interaction', ts=None, from_pop_type=from_pop_type, to_pop_type=to_pop_type)
+        from_pops = [name for name, pop_spec in self.pops.items() if pop_spec["type"] == from_pop_type]
+        to_pops = [name for name, pop_spec in self.pops.items() if pop_spec["type"] == to_pop_type]
+        interpop = TimeDependentConnections(code_name, full_name, tvec=self.tvec, from_pops=from_pops, to_pops=to_pops, interpop_type="interaction", ts=None, from_pop_type=from_pop_type, to_pop_type=to_pop_type)
         interpop.write_units = True
         interpop.write_assumption = True
         interpop.write_uncertainty = True
@@ -810,15 +812,15 @@ class ProjectData(sc.prettyobj):
 
         # TODO - can modify _read_pops() and _write_pops() if there are more population attributes
         tables = read_tables(sheet)[0]
-        assert len(tables) == 1, 'Population Definitions page should only contain one table'
+        assert len(tables) == 1, "Population Definitions page should only contain one table"
 
         self.pops = sc.odict()
-        assert cell_get_string(tables[0][0][0]).lower() == 'abbreviation'
-        assert cell_get_string(tables[0][0][1]).lower() == 'full name'
+        assert cell_get_string(tables[0][0][0]).lower() == "abbreviation"
+        assert cell_get_string(tables[0][0][1]).lower() == "full name"
 
         # If pop typ column exists, check the heading is correct
         if len(tables[0][0]) > 2:
-            assert cell_get_string(tables[0][0][2]).lower() == 'population type'
+            assert cell_get_string(tables[0][0][2]).lower() == "population type"
 
         for row in tables[0][1:]:
 
@@ -835,7 +837,7 @@ class ProjectData(sc.prettyobj):
             if len(row) > 2 and row[2].value is not None:
                 poptype = cell_get_string(row[2])
 
-            self.pops[pop_name] = {'label': label, 'type': poptype}
+            self.pops[pop_name] = {"label": label, "type": poptype}
 
     def _write_pops(self) -> None:
         """
@@ -844,28 +846,28 @@ class ProjectData(sc.prettyobj):
         """
 
         sheet = self._book.add_worksheet("Population Definitions")
-        sheet.set_tab_color('#FFC000')  # this tab is orange
+        sheet.set_tab_color("#FFC000")  # this tab is orange
         widths = dict()
 
         current_row = 0
-        sheet.write(current_row, 0, 'Abbreviation', self._formats["center_bold"])
-        update_widths(widths, 0, 'Abbreviation')
-        sheet.write(current_row, 1, 'Full Name', self._formats["center_bold"])
-        update_widths(widths, 1, 'Abbreviation')
-        sheet.write(current_row, 2, 'Population type', self._formats["center_bold"])
-        update_widths(widths, 2, 'Population type')
+        sheet.write(current_row, 0, "Abbreviation", self._formats["center_bold"])
+        update_widths(widths, 0, "Abbreviation")
+        sheet.write(current_row, 1, "Full Name", self._formats["center_bold"])
+        update_widths(widths, 1, "Abbreviation")
+        sheet.write(current_row, 2, "Population type", self._formats["center_bold"])
+        update_widths(widths, 2, "Population type")
 
         for name, content in self.pops.items():
             current_row += 1
-            sheet.write(current_row, 0, name, self._formats['unlocked'])
+            sheet.write(current_row, 0, name, self._formats["unlocked"])
             update_widths(widths, 0, name)
-            sheet.write(current_row, 1, content['label'], self._formats['unlocked'])
-            update_widths(widths, 1, content['label'])
-            sheet.write(current_row, 2, content['type'], self._formats['not_required'])
-            update_widths(widths, 2, content['type'])
+            sheet.write(current_row, 1, content["label"], self._formats["unlocked"])
+            update_widths(widths, 1, content["label"])
+            sheet.write(current_row, 2, content["type"], self._formats["not_required"])
+            update_widths(widths, 2, content["type"])
 
             self._references[name] = "='%s'!%s" % (sheet.name, xlrc(current_row, 0, True, True))
-            self._references[content['label']] = "='%s'!%s" % (sheet.name, xlrc(current_row, 1, True, True))  # Reference to the full name
+            self._references[content["label"]] = "='%s'!%s" % (sheet.name, xlrc(current_row, 1, True, True))  # Reference to the full name
 
         apply_widths(sheet, widths)
 
@@ -876,10 +878,10 @@ class ProjectData(sc.prettyobj):
         """
 
         tables, start_rows = read_tables(sheet)
-        assert len(tables) % 3 == 0, 'There should be 3 subtables for every transfer'
+        assert len(tables) % 3 == 0, "There should be 3 subtables for every transfer"
         self.transfers = []
         for i in range(0, len(tables), 3):
-            self.transfers.append(TimeDependentConnections.from_tables(tables[i:i + 3], 'transfer'))
+            self.transfers.append(TimeDependentConnections.from_tables(tables[i : i + 3], "transfer"))
 
     def _write_transfers(self) -> None:
         """
@@ -893,7 +895,7 @@ class ProjectData(sc.prettyobj):
             return
 
         sheet = self._book.add_worksheet("Transfers")
-        sheet.set_tab_color('#808080')
+        sheet.set_tab_color("#808080")
         # sheet.hide()
         widths = dict()
         next_row = 0
@@ -907,10 +909,10 @@ class ProjectData(sc.prettyobj):
 
         """
         tables, start_rows = read_tables(sheet)
-        assert len(tables) % 3 == 0, 'There should be 3 subtables for every transfer'
+        assert len(tables) % 3 == 0, "There should be 3 subtables for every transfer"
         self.interpops = []
         for i in range(0, len(tables), 3):
-            self.interpops.append(TimeDependentConnections.from_tables(tables[i:i + 3], 'interaction'))
+            self.interpops.append(TimeDependentConnections.from_tables(tables[i : i + 3], "interaction"))
         return
 
     def _write_interpops(self) -> None:
@@ -924,7 +926,7 @@ class ProjectData(sc.prettyobj):
             return
 
         sheet = self._book.add_worksheet("Interactions")
-        sheet.set_tab_color('#808080')
+        sheet.set_tab_color("#808080")
         widths = dict()
         next_row = 0
         for interpop in self.interpops:
@@ -951,8 +953,8 @@ class ProjectData(sc.prettyobj):
                 next_row = self.tdve[code_name].write(sheet, next_row, self._formats, references=self._references, widths=widths)
 
             if has_editable_content:
-                sheet.set_tab_color('#92D050')
+                sheet.set_tab_color("#92D050")
             else:
-                sheet.set_tab_color('#808080')
+                sheet.set_tab_color("#808080")
 
             apply_widths(self._book.get_worksheet_by_name(sheet_name), widths)
