@@ -11,6 +11,8 @@ import ast
 import numpy as np
 from functools import reduce
 
+__all__ = ["parse_function"]
+
 
 def sdiv(numerator, denominator):
     """
@@ -74,24 +76,7 @@ def vector_max(*args):
 
 
 # Only calls to functions in the dict below will be permitted
-supported_functions = {
-    'max': vector_max,
-    'min': vector_min,
-    'exp': np.exp,
-    'floor': np.floor,
-    'SRC_POP_AVG': None,
-    'TGT_POP_AVG': None,
-    'SRC_POP_SUM': None,
-    'TGT_POP_SUM': None,
-    'pi': np.pi,
-    'cos': np.cos,
-    'sin': np.sin,
-    'sqrt': np.sqrt,
-    'ln': np.log,
-    'rand': np.random.rand,
-    'randn': np.random.randn,
-    'sdiv': sdiv
-}
+supported_functions = {"max": vector_max, "min": vector_min, "exp": np.exp, "floor": np.floor, "SRC_POP_AVG": None, "TGT_POP_AVG": None, "SRC_POP_SUM": None, "TGT_POP_SUM": None, "pi": np.pi, "cos": np.cos, "sin": np.sin, "sqrt": np.sqrt, "ln": np.log, "rand": np.random.rand, "randn": np.random.randn, "sdiv": sdiv}
 
 
 class _DivTransformer(ast.NodeTransformer):
@@ -114,7 +99,7 @@ class _DivTransformer(ast.NodeTransformer):
             node.right = rhs
             return node
 
-        name = ast.Name('sdiv', ast.Load())
+        name = ast.Name("sdiv", ast.Load())
         args = [lhs, rhs]
         kwargs = []
         return ast.Call(name, args, kwargs)
@@ -156,17 +141,17 @@ def parse_function(fcn_str: str) -> tuple:
     # Where dep_list corresponds to a list of keys for
     # the dict that needs to be passed to fcn()
     # supported_functions is a dict mapping ast names to functors imported in the namespace of this file
-    assert '__' not in fcn_str, 'Cannot use double underscores in functions'
+    assert "__" not in fcn_str, "Cannot use double underscores in functions"
     assert len(fcn_str) < 1800  # Function string must be less than 1800 characters
-    fcn_str = fcn_str.replace(':', '___')
-    fcn_ast = ast.parse(fcn_str, mode='eval')
+    fcn_str = fcn_str.replace(":", "___")
+    fcn_ast = ast.parse(fcn_str, mode="eval")
     fcn_ast = _DivTransformer().visit(fcn_ast)
     fcn_ast = ast.fix_missing_locations(fcn_ast)
     dep_list = []
     for node in ast.walk(fcn_ast):
         if isinstance(node, ast.Name) and node.id not in supported_functions:
             dep_list.append(node.id)
-        elif isinstance(node, ast.Call) and hasattr(node, 'func') and hasattr(node.func, 'id'):
+        elif isinstance(node, ast.Call) and hasattr(node, "func") and hasattr(node.func, "id"):
             assert node.func.id in supported_functions, "Only calls to supported functions are allowed"
     compiled_code = compile(fcn_ast, filename="<ast>", mode="eval")
 
@@ -177,10 +162,10 @@ def parse_function(fcn_str: str) -> tuple:
 
 
 # Example usage below - This can be moved to documentation later.
-if __name__ == '__main__':
-    f_string = 'exp(x)+y**2'
+if __name__ == "__main__":
+    f_string = "exp(x)+y**2"
     fcn, dep_list = parse_function(f_string)
     print(dep_list)
-    deps = {'x': 1, 'y': 2}
+    deps = {"x": 1, "y": 2}
     print(fcn(**deps))
     print(fcn(x=1, y=3))  # The use of **deps means you can write out keyword arguments for `fcn` directly
