@@ -11,7 +11,7 @@ import os
 import re
 import time
 import zlib
-from bisect import bisect
+from bisect import bisect_right, bisect_left
 from datetime import datetime
 from functools import partial
 from pathlib import Path
@@ -23,6 +23,21 @@ from tqdm import tqdm
 import sciris as sc
 from .system import logger
 
+__all__ = [
+    "NamedItem",
+    "NDict",
+    "TimeSeries",
+    "Quiet",
+    "parent_dir",
+    "evaluate_plot_string",
+    "format_duration",
+    "nested_loop",
+    "fast_gitinfo",
+    "datetime_to_year",
+    "parallel_progress",
+    "start_logging",
+]
+
 
 def parent_dir() -> Path:
     """
@@ -39,7 +54,7 @@ def parent_dir() -> Path:
     return Path(inspect.stack()[1][1]).parent
 
 
-class NamedItem():
+class NamedItem:
     def __init__(self, name: str = None):
         """
         NamedItem constructor
@@ -49,7 +64,7 @@ class NamedItem():
         :param name:
         """
         if name is None:
-            name = '<unnamed>'
+            name = "<unnamed>"
         self.name = name
         self.created = sc.now(utc=True)
         self.modified = sc.now(utc=True)
@@ -113,7 +128,7 @@ class NDict(sc.odict):
         return self[new]
 
 
-class TimeSeries(object):
+class TimeSeries:
     """
     Class to store time-series data
 
@@ -131,11 +146,11 @@ class TimeSeries(object):
 
     """
 
-    def __init__(self, t=None, vals=None, units: str = None, assumption: float = None, sigma: float = None):
-        t = sc.promotetolist(t) if t is not None else list()
-        vals = sc.promotetolist(vals) if vals is not None else list()
+    # Use slots here to guarantee that __deepcopy__() and __eq__() only have to check these
+    # specific fields - otherwise, would need to do a more complex recursive dict comparison
+    __slots__ = ["t", "vals", "units", "assumption", "sigma", "_sampled"]
 
-        assert len(t) == len(vals)
+    def __init__(self, t=None, vals=None, units: str = None, assumption: float = None, sigma: float = None):
 
         self.t = []  #: Sorted array of time points. Normally interacted with via methods like :meth:`insert()`
         self.vals = []  #: Time-specific values - indices correspond to ``self.t``
@@ -144,13 +159,28 @@ class TimeSeries(object):
         self.sigma = sigma  #: Uncertainty value, assumed to be a standard deviation
         self._sampled = False  #: Flag to indicate whether sampling has been performed. Once sampling has been performed, cannot sample again
 
-        for tx, vx in zip(t, vals):
-            self.insert(tx, vx)
+        # Using insert() means that array/list inputs containing None or duplicate entries will
+        # be sanitized via insert()
+        self.insert(t, vals)
 
     def __repr__(self):
         output = sc.prepr(self)
         return output
 
+    def __eq__(self, other):
+        """
+        Check TimeSeries equality
+
+        Two TimeSeries instances are equal if all of their attributes are equal. This is easy to
+        implement because `==` is directly defined for all of the attribute types (lists and scalars)
+        and due to `__slots__` there are guaranteed not to be any other attributes
+
+        :param other:
+        :return:
+        """
+
+        return all(getattr(self, x) == getattr(other, x) for x in self.__slots__)
+
     def __deepcopy__(self, memodict={}):
         new = TimeSeries.__new__(TimeSeries)
         new.t = self.t.copy()
@@ -161,6 +191,23 @@ class TimeSeries(object):
         new._sampled = self._sampled
         return new
 
+    def __getstate__(self):
+        return dict([(k, getattr(self, k, None)) for k in self.__slots__])
+
+    def __setstate__(self, data):
+
+        if "format" in data:
+            # 'format' was changed to 'units' but the attribute was not dropped, however now this is a
+            # hard error because of the switch to __slots__ so we need to make sure it gets removed.
+            # This can't be done as a Migration because a Migration expects an instance of the
+            data = sc.dcp(data)
+            if "units" not in data:
+                data["units"] = data["format"]
+            del data["format"]
+
+        for k, v in data.items():
+            setattr(self, k, v)
+
     # The operators for + and * below are prototypes but haven't been added
     # The reason is because they add a lot of complexity to the usage of the class
     # but it's not clear whether that complexity is worth it for the benefits it brings.
@@ -329,25 +376,36 @@ class TimeSeries(object):
         :param v: Value to insert. If ``None``, this function will return immediately without doing anything
 
         """
-        if isinstance(t, list):
-            assert isinstance(v, list) and len(t) == len(v), 'Cannot insert non-matching lengths or types  of time and values %s and %s' % (t, v)
+
+        # Check if inputs are iterable
+        iterable_input = True
+        try:
+            assert len(t) == len(v), "Cannot insert non-matching lengths or types of time and values %s and %s" % (t, v)
+        except TypeError:
+            iterable_input = False
+
+        # If inputs are iterable, call insert() for each zipped item
+        if iterable_input:
             for ti, vi in zip(t, v):
                 self.insert(ti, vi)
+            return
+
+        if v is None:  # Can't cast a None to a float, so just skip it
+            return
+
+        v = float(v)  # Convert input to float
+
+        if t is None:  # Store the value in the assumption
+            self.assumption = v
+            return
+
+        idx = bisect_left(self.t, t)
+        if idx < len(self.t) and self.t[idx] == t:
+            # Overwrite an existing entry
+            self.vals[idx] = v
         else:
-            if v is None:  # Can't cast a None to a float, just skip it
-                return
-
-            v = float(v)  # Convert input to float
-
-            if t is None:
-                self.assumption = v
-            elif t in self.t:
-                idx = self.t.index(t)
-                self.vals[idx] = v
-            else:
-                idx = bisect(self.t, t)
-                self.t.insert(idx, t)
-                self.vals.insert(idx, v)
+            self.t.insert(idx, t)
+            self.vals.insert(idx, v)
 
     def get(self, t) -> float:
         """
@@ -413,7 +471,7 @@ class TimeSeries(object):
             del self.t[idx]
             del self.vals[idx]
         else:
-            raise Exception('Item not found')
+            raise Exception("Item not found")
 
     def remove_before(self, t_remove) -> None:
         """
@@ -453,7 +511,7 @@ class TimeSeries(object):
             if t_remove[0] < tval < t_remove[1]:
                 self.remove(tval)
 
-    def interpolate(self, t2: np.array, method='linear', **kwargs) -> np.array:
+    def interpolate(self, t2: np.array, method="linear", **kwargs) -> np.array:
         """
         Return interpolated values
 
@@ -502,16 +560,16 @@ class TimeSeries(object):
         idx = ~np.isnan(t1) & ~np.isnan(v1)
         t1, v1 = t1[idx], v1[idx]
         if t1.size == 0:
-            raise Exception('No time points remained after removing NaNs from the TimeSeries')
+            raise Exception("No time points remained after removing NaNs from the TimeSeries")
         elif t1.size == 1:
             return np.full(t2.shape, v1[0])
 
         # # Finally, perform interpolation
         if sc.isstring(method):
-            if method == 'linear':
+            if method == "linear":
                 # Default linear interpolation
                 return np.interp(t2, t1, v1, left=v1[0], right=v1[-1])
-            elif method == 'pchip':
+            elif method == "pchip":
                 # Legacy pchip interpolation
                 f = scipy.interpolate.PchipInterpolator(t1, v1, axis=0, extrapolate=False)
                 y2 = np.zeros(t2.shape)
@@ -519,8 +577,8 @@ class TimeSeries(object):
                 y2[t2 < t1[0]] = v1[0]
                 y2[t2 > t1[-1]] = v1[-1]
                 return y2
-            elif method == 'previous':
-                return scipy.interpolate.interp1d(t1, v1, kind='previous', copy=False, assume_sorted=True, bounds_error=False, fill_value=(v1[0], v1[-1]))(t2)
+            elif method == "previous":
+                return scipy.interpolate.interp1d(t1, v1, kind="previous", copy=False, assume_sorted=True, bounds_error=False, fill_value=(v1[0], v1[-1]))(t2)
             else:
                 raise Exception('Unknown interpolation type - must be one of "linear", "pchip", or "previous"')
 
@@ -543,7 +601,7 @@ class TimeSeries(object):
         """
 
         if self._sampled:
-            raise Exception('Sampling has already been performed - can only sample once')
+            raise Exception("Sampling has already been performed - can only sample once")
 
         new = self.copy()
         if self.sigma is not None:
@@ -591,14 +649,14 @@ def evaluate_plot_string(plot_string: str):
     :return: Evaluated expression, the same as if it has originally been entered in a .py file
     """
 
-    if '{' in plot_string or '[' in plot_string:
+    if "{" in plot_string or "[" in plot_string:
         # Evaluate the string to set lists and dicts - do at least a little validation
-        assert '__' not in plot_string, 'Cannot use double underscores in functions'
+        assert "__" not in plot_string, "Cannot use double underscores in functions"
         assert len(plot_string) < 1800  # Function string must be less than 1800 characters
-        fcn_ast = ast.parse(plot_string, mode='eval')
+        fcn_ast = ast.parse(plot_string, mode="eval")
         for node in ast.walk(fcn_ast):
             if not (node is fcn_ast):
-                assert isinstance(node, ast.Dict) or isinstance(node, ast.Str) or isinstance(node, ast.List) or isinstance(node, ast.Load), 'Only allowed to initialize lists and dicts of strings here'
+                assert isinstance(node, ast.Dict) or isinstance(node, ast.Str) or isinstance(node, ast.List) or isinstance(node, ast.Load), "Only allowed to initialize lists and dicts of strings here"
         compiled_code = compile(fcn_ast, filename="<ast>", mode="eval")
         return eval(compiled_code)
     else:
@@ -640,30 +698,30 @@ def format_duration(t: float, pluralize=False) -> str:
     # First decide the base units
     if t >= 1.0:
         base_scale = 1
-        timescale = 'year'
+        timescale = "year"
     elif t >= 1 / 12:
         base_scale = 1 / 12
-        timescale = 'month'
+        timescale = "month"
     elif t >= 1 / 26:
         base_scale = 1 / 26
-        timescale = 'fortnight'
+        timescale = "fortnight"
     elif t >= 1 / 52:
         base_scale = 1 / 52
-        timescale = 'week'
+        timescale = "week"
     else:
         base_scale = 1 / 365
-        timescale = 'day'
+        timescale = "day"
 
     # Then, work out how many of the base unit there are
     converted_t = t / base_scale
 
     # If there is only one of the base unit, then return the timescale as the final string
     if abs(converted_t - 1.0) < 1e-5:
-        return (timescale + 's') if pluralize else timescale
+        return (timescale + "s") if pluralize else timescale
     elif converted_t % 1 < 1e-3:  # If it's sufficiently close to an integer, show it as an integer
-        return '%d %ss' % (converted_t, timescale)
+        return "%d %ss" % (converted_t, timescale)
     else:
-        return '%s %ss' % (sc.sigfig(converted_t, keepints=True, sigfigs=3), timescale)
+        return "%s %ss" % (sc.sigfig(converted_t, keepints=True, sigfigs=3), timescale)
 
 
 def nested_loop(inputs, loop_order):
@@ -724,8 +782,8 @@ def fast_gitinfo(path):
         # First, get the .git directory
         curpath = os.path.abspath(path)
         while curpath:
-            if os.path.exists(os.path.join(curpath, '.git')):
-                gitdir = os.path.join(curpath, '.git')
+            if os.path.exists(os.path.join(curpath, ".git")):
+                gitdir = os.path.join(curpath, ".git")
                 break
             else:
                 parent, _ = os.path.split(curpath)
@@ -734,37 +792,37 @@ def fast_gitinfo(path):
                 else:
                     curpath = parent
         else:
-            raise Exception('Could not find .git directory')
+            raise Exception("Could not find .git directory")
 
         # Then, get the branch and commit
-        with open(os.path.join(gitdir, 'HEAD'), 'r') as f1:
+        with open(os.path.join(gitdir, "HEAD"), "r") as f1:
             ref = f1.read()
-            if ref.startswith('ref:'):
-                refdir = ref.split(' ')[1].strip()  # The path to the file with the commit
-                gitbranch = refdir.replace('refs/heads/', '')  # / is always used (not os.sep)
-                with open(os.path.join(gitdir, refdir), 'r') as f2:
+            if ref.startswith("ref:"):
+                refdir = ref.split(" ")[1].strip()  # The path to the file with the commit
+                gitbranch = refdir.replace("refs/heads/", "")  # / is always used (not os.sep)
+                with open(os.path.join(gitdir, refdir), "r") as f2:
                     githash = f2.read().strip()  # The hash of the commit
             else:
-                gitbranch = 'Detached head (no branch)'
+                gitbranch = "Detached head (no branch)"
                 githash = ref.strip()
 
         # Now read the time from the commit
-        compressed_contents = open(os.path.join(gitdir, 'objects', githash[0:2], githash[2:]), 'rb').read()
+        compressed_contents = open(os.path.join(gitdir, "objects", githash[0:2], githash[2:]), "rb").read()
         decompressed_contents = zlib.decompress(compressed_contents).decode()
-        for line in decompressed_contents.split('\n'):
-            if line.startswith('author'):
-                _re_actor_epoch = re.compile(r'^.+? (.*) (\d+) ([+-]\d+).*$')
+        for line in decompressed_contents.split("\n"):
+            if line.startswith("author"):
+                _re_actor_epoch = re.compile(r"^.+? (.*) (\d+) ([+-]\d+).*$")
                 m = _re_actor_epoch.search(line)
                 actor, epoch, offset = m.groups()
                 t = time.gmtime(int(epoch))
                 gitdate = time.strftime("%Y-%m-%d %H:%M:%S UTC", t)
 
     except Exception:
-        gitbranch = 'Git branch N/A'
-        githash = 'Git hash N/A'
-        gitdate = 'Git date N/A'
+        gitbranch = "Git branch N/A"
+        githash = "Git hash N/A"
+        gitdate = "Git date N/A"
 
-    output = {'branch': gitbranch, 'hash': githash, 'date': gitdate}  # Assemble outupt
+    output = {"branch": gitbranch, "hash": githash, "date": gitdate}  # Assemble outupt
     return output
 
 
@@ -851,7 +909,7 @@ def parallel_progress(fcn, inputs, num_workers=None, show_progress=True) -> list
     return results
 
 
-class Quiet():
+class Quiet:
     """
     Atomica quiet context
 
@@ -915,7 +973,7 @@ def start_logging(fname: str) -> None:
     """
 
     for handler in logger.handlers:
-        if handler.name == 'atomica_file_handler':
+        if handler.name == "atomica_file_handler":
             # Logging has already been started, so we can return immediately
             # Otherwise, output could be logged multiple times
             return
@@ -923,20 +981,20 @@ def start_logging(fname: str) -> None:
     from .version import gitinfo, version, versiondate  # Avoid circular import
 
     sc.makefilepath(fname)
-    h = logging.FileHandler(fname, mode='w')
-    h.set_name('atomica_file_handler')
-    fmt = logging.Formatter("%(asctime)-20s %(message)s", datefmt='%d-%m-%y %H:%M:%S')
+    h = logging.FileHandler(fname, mode="w")
+    h.set_name("atomica_file_handler")
+    fmt = logging.Formatter("%(asctime)-20s %(message)s", datefmt="%d-%m-%y %H:%M:%S")
     h.setFormatter(fmt)
 
     logger.addHandler(h)
 
-    logger.critical(f'Atomica log file: {os.path.abspath(fname)}')
-    logger.critical('-' * 80)
-    logger.critical('Atomica %s (%s) -- (c) the Atomica development team' % (version, versiondate))  # Log with the highest level to make sure it appears in the log
-    if gitinfo['branch'] != 'N/A':
-        logger.critical('git branch: %s (%s)' % (gitinfo['branch'], gitinfo['hash'][0:8]))
+    logger.critical(f"Atomica log file: {os.path.abspath(fname)}")
+    logger.critical("-" * 80)
+    logger.critical("Atomica %s (%s) -- (c) the Atomica development team" % (version, versiondate))  # Log with the highest level to make sure it appears in the log
+    if gitinfo["branch"] != "N/A":
+        logger.critical("git branch: %s (%s)" % (gitinfo["branch"], gitinfo["hash"][0:8]))
     logger.critical(datetime.now())
-    logger.critical('-' * 80)
+    logger.critical("-" * 80)
 
     # We also want uncaught exceptions (in the main process) to appear in the log
     # Based on https://stackoverflow.com/a/57587758 by Nabs
@@ -948,8 +1006,10 @@ def start_logging(fname: str) -> None:
             local_args = run_func()
             hook_func(*local_args)
             return run_func(*args, **kwargs)
+
         return inner
 
     import sys
+
     sys.exc_info = attach_hook(log_exception, sys.exc_info)
     sys.excepthook = log_exception
